
'''
This file is automatically generated; Do not edit it. :)
'''
VERSION_INFO = {
    'final': True,
    'version': '1.15.2',
    'branch_nick': 'patch-0-1_15_2_preparation',
    'revision_id': '13db4e6b7409b1a2ad8fc00f0dcdd64a0765cf95',
    'revno': 6289
}
